#include <SoftwareSerial.h>
#include <CompactQik2s9v1.h>

#define xbeeRx    A0           // DOUT to A0
#define xbeeTx    A1           // DIN to A1
#define rxPin     5            // Qik motor Rx
#define txPin     6            // Qik motor Tx
#define rstPin    7            // Qik motor reset

byte topSpeed = 96;

SoftwareSerial motorSerial (rxPin, txPin);
CompactQik2s9v1 motor = CompactQik2s9v1(&motorSerial, rstPin);
SoftwareSerial Xbee (xbeeRx, xbeeTx);    //After motor serial

boolean motorsOn = false;

void setup() {
  Serial.begin(9600);
  motorSerial.begin(9600);
  motor.begin();
  motor.stopBothMotors();
  Xbee.begin(9600);          // Place last
  delay(200);
  pinMode(13, OUTPUT);       // Show ready status
  digitalWrite(13, HIGH);
}

void loop() {
  readXbee();
  delay(50);
}

void readXbee() {
  byte bytehigh, bytelow;           // High and low bytes of bearing
  char pitch, roll;                 // Pitch and roll (signed values)
  int bearing;                      // Full bearing  
   
  if(Xbee.available() == 5) { 
    Xbee.read();                    // Throw away first
    bytehigh = Xbee.read();         // Store other values
    bytelow = Xbee.read();            
    pitch = Xbee.read();              
    roll = Xbee.read(); 
    bearing = ((bytehigh<<8) + bytelow) / 10;  // Full bearing
    //print_data(bearing, pitch, roll);          // Print data
    controlMotor(pitch, roll);                 // Control motors
  }
}

void controlMotor(char pitch, char roll) {
  topSpeed = min(abs(pitch)*3, 127);
  // or use: topSpeed = constrain(map(abs(pitch), 5, 50, 0, 127), 0, 127);
  Serial.println(topSpeed);
  if(pitch > 5) {
    motorsReverse(topSpeed, roll);
  } else if(pitch < -5) {
    motorsForward(topSpeed, roll);
  } else {
    motorsStop();
  }
}

//  motor0 = left, motor1 = right
void motorsStop() {
  motor.motor0Forward(0);
  motor.motor1Forward(0);
}
void motorsForward(byte motorSpeed, char turn) {
  if(turn > 20) {
    motor.motor0Forward(motorSpeed);
    motor.motor1Forward(0);
  } else if(turn < -20) {
    motor.motor0Forward(0);
    motor.motor1Forward(motorSpeed);
  } else {
    motor.motor0Forward(motorSpeed);
    motor.motor1Forward(motorSpeed);    
  } 
}
void motorsReverse(byte motorSpeed, char turn) {
  motor.motor0Reverse(motorSpeed);
  motor.motor1Reverse(motorSpeed);
}

// Print data to Serial Monitor window
void print_data(int bearing, int pitch, int roll) {
  Serial.print(bearing, DEC); 
  Serial.print(":");
  Serial.print(pitch, DEC);
  Serial.print(":");
  Serial.println(roll, DEC);
}

// Used for testing; displays raw values received by XBee
void readRaw() {
  Serial.print(Xbee.read());
  Serial.print(":");
  Serial.print(Xbee.read());
  Serial.print(":");
  Serial.print(Xbee.read());
  Serial.print(":");    
  Serial.print(Xbee.read());
  Serial.print(":");    
  Serial.println(Xbee.read());
}
